@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCmdIL.c,v 1.15 1999/08/10 17:35:18 redman Exp $
+ * RCS: @(#) $Id: tclCmdIL.c,v 1.23 2000/01/26 03:37:40 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -30,6 +30,7 @@
 
 typedef struct SortElement {
     Tcl_Obj *objPtr;			/* Object being sorted. */
+    int count;				/* number of same elements in list */
     struct SortElement *nextPtr;        /* Next element in the list, or
 					 * NULL for end of list. */
 } SortElement;
@@ -465,7 +466,7 @@ Tcl_InfoObjCmd(clientData, interp, objc, objv)
  *          info args procName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -527,7 +528,7 @@ InfoArgsCmd(dummy, interp, objc, objv)
  *          info body procName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -562,9 +563,12 @@ InfoBodyCmd(dummy, interp, objc, objv)
     }
 
     /*
-     * we need to check if the body from this procedure had been generated
-     * from a precompiled body. If that is the case, then the bodyPtr's
-     * string representation is bogus, since sources are not available.
+     * We should not return a bytecompiled body.  If it is precompiled,
+     * then the bodyPtr's string representation is bogus, since sources
+     * are not available.  If it was just a bytecompiled body, then it
+     * is likely to not be of any use to the caller, as it was compiled
+     * for a separate procedure context [Bug: 3412], and noone else can
+     * reasonably use it.
      * In order to make sure that later manipulations of the object do not
      * invalidate the internal representation, we make a copy of the string
      * representation and return that one, instead.
@@ -573,11 +577,7 @@ InfoBodyCmd(dummy, interp, objc, objv)
     bodyPtr = procPtr->bodyPtr;
     resultPtr = bodyPtr;
     if (bodyPtr->typePtr == &tclByteCodeType) {
-        ByteCode *codePtr = (ByteCode *) bodyPtr->internalRep.otherValuePtr;
-
-        if (codePtr->flags & TCL_BYTECODE_PRECOMPILED) {
-            resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
-        }
+	resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
     }
     
     Tcl_SetObjResult(interp, resultPtr);
@@ -596,7 +596,7 @@ InfoBodyCmd(dummy, interp, objc, objv)
  *          info cmdcount
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -638,7 +638,7 @@ InfoCmdCountCmd(dummy, interp, objc, objv)
  *          info commands ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -654,8 +654,8 @@ InfoCommandsCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    char *cmdName, *pattern;
-    CONST char *simplePattern;
+    char *cmdName;
+    CONST char *pattern, *simplePattern;
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
     Namespace *nsPtr;
@@ -765,7 +765,7 @@ InfoCommandsCmd(dummy, interp, objc, objv)
  *          info complete command
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -807,7 +807,7 @@ InfoCompleteCmd(dummy, interp, objc, objv)
  *          info default procName arg varName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -891,7 +891,7 @@ InfoDefaultCmd(dummy, interp, objc, objv)
  *          info exists varName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -937,7 +937,7 @@ InfoExistsCmd(dummy, interp, objc, objv)
  *          info globals ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1003,7 +1003,7 @@ InfoGlobalsCmd(dummy, interp, objc, objv)
  *          info hostname
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1047,7 +1047,7 @@ InfoHostnameCmd(dummy, interp, objc, objv)
  *          info level ?number?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1121,7 +1121,7 @@ InfoLevelCmd(dummy, interp, objc, objv)
  *          info library
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1166,7 +1166,7 @@ InfoLibraryCmd(dummy, interp, objc, objv)
  *          info loaded ?interp?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1211,7 +1211,7 @@ InfoLoadedCmd(dummy, interp, objc, objv)
  *          info locals ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1340,7 +1340,7 @@ AppendLocals(interp, listPtr, pattern, includeLinks)
  *          info nameofexecutable
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1383,7 +1383,7 @@ InfoNameOfExecutableCmd(dummy, interp, objc, objv)
  *          info patchlevel
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1420,14 +1420,17 @@ InfoPatchLevelCmd(dummy, interp, objc, objv)
  *
  * InfoProcsCmd --
  *
- *      Called to implement the "info procs" command that returns the
- *      procedures in the current namespace that match an optional pattern.
- *      Handles the following syntax:
+ *	Called to implement the "info procs" command that returns the
+ *	list of procedures in the interpreter that match an optional pattern.
+ *	The pattern, if any, consists of an optional sequence of namespace
+ *	names separated by "::" qualifiers, which is followed by a
+ *	glob-style pattern that restricts which commands are returned.
+ *	Handles the following syntax:
  *
  *          info procs ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1443,50 +1446,127 @@ InfoProcsCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    char *cmdName, *pattern;
-    Namespace *currNsPtr = (Namespace *) Tcl_GetCurrentNamespace(interp);
+    char *cmdName;
+    CONST char *pattern, *simplePattern;
+    Namespace *nsPtr;
+#ifdef INFO_PROCS_SEARCH_GLOBAL_NS
+    Namespace *globalNsPtr = (Namespace *) Tcl_GetGlobalNamespace(interp);
+#endif
+    Namespace *currNsPtr   = (Namespace *) Tcl_GetCurrentNamespace(interp);
+    Tcl_Obj *listPtr, *elemObjPtr;
+    int specificNsInPattern = 0;  /* Init. to avoid compiler warning. */
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
     Command *cmdPtr, *realCmdPtr;
-    Tcl_Obj *listPtr;
+
+    /*
+     * Get the pattern and find the "effective namespace" in which to
+     * list procs.
+     */
 
     if (objc == 2) {
-        pattern = NULL;
+	simplePattern = NULL;
+	nsPtr = currNsPtr;
+	specificNsInPattern = 0;
     } else if (objc == 3) {
-        pattern = Tcl_GetString(objv[2]);
+	/*
+	 * From the pattern, get the effective namespace and the simple
+	 * pattern (no namespace qualifiers or ::'s) at the end. If an
+	 * error was found while parsing the pattern, return it. Otherwise,
+	 * if the namespace wasn't found, just leave nsPtr NULL: we will
+	 * return an empty list since no commands there can be found.
+	 */
+
+	Namespace *dummy1NsPtr, *dummy2NsPtr;
+
+	pattern = Tcl_GetString(objv[2]);
+	TclGetNamespaceForQualName(interp, pattern, (Namespace *) NULL,
+		/*flags*/ 0, &nsPtr, &dummy1NsPtr, &dummy2NsPtr,
+		&simplePattern);
+
+	if (nsPtr != NULL) {	/* we successfully found the pattern's ns */
+	    specificNsInPattern = (strcmp(simplePattern, pattern) != 0);
+	}
     } else {
         Tcl_WrongNumArgs(interp, 2, objv, "?pattern?");
         return TCL_ERROR;
     }
 
     /*
-     * Scan through the current namespace's command table and return a list
-     * of all procs that match the pattern.
+     * Scan through the effective namespace's command table and create a
+     * list with all procs that match the pattern. If a specific
+     * namespace was requested in the pattern, qualify the command names
+     * with the namespace name.
      */
-    
+
     listPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
-    for (entryPtr = Tcl_FirstHashEntry(&currNsPtr->cmdTable, &search);
-            entryPtr != NULL;
-            entryPtr = Tcl_NextHashEntry(&search)) {
-        cmdName = Tcl_GetHashKey(&currNsPtr->cmdTable, entryPtr);
-        cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+    if (nsPtr != NULL) {
+	entryPtr = Tcl_FirstHashEntry(&nsPtr->cmdTable, &search);
+	while (entryPtr != NULL) {
+	    cmdName = Tcl_GetHashKey(&nsPtr->cmdTable, entryPtr);
+	    if ((simplePattern == NULL)
+	            || Tcl_StringMatch(cmdName, simplePattern)) {
+		cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+
+		if (specificNsInPattern) {
+		    elemObjPtr = Tcl_NewObj();
+		    Tcl_GetCommandFullName(interp, (Tcl_Command) cmdPtr,
+		            elemObjPtr);
+		} else {
+		    elemObjPtr = Tcl_NewStringObj(cmdName, -1);
+		}
+
+		realCmdPtr = (Command *)
+		    TclGetOriginalCommand((Tcl_Command) cmdPtr);
+
+		if (TclIsProc(cmdPtr)
+		        || ((realCmdPtr != NULL) && TclIsProc(realCmdPtr))) {
+		    Tcl_ListObjAppendElement(interp, listPtr, elemObjPtr);
+		}
+	    }
+	    entryPtr = Tcl_NextHashEntry(&search);
+	}
 
 	/*
-	 * If the command isn't itself a proc, it still might be an
-	 * imported command that points to a "real" proc in a different
-	 * namespace.
+	 * If the effective namespace isn't the global :: namespace, and a
+	 * specific namespace wasn't requested in the pattern, then add in
+	 * all global :: procs that match the simple pattern. Of course,
+	 * we add in only those procs that aren't hidden by a proc in
+	 * the effective namespace.
 	 */
 
-	realCmdPtr = (Command *) TclGetOriginalCommand(
-	        (Tcl_Command) cmdPtr);
-        if (TclIsProc(cmdPtr)
-	        || ((realCmdPtr != NULL) && TclIsProc(realCmdPtr))) {
-            if ((pattern == NULL) || Tcl_StringMatch(cmdName, pattern)) {
-                Tcl_ListObjAppendElement(interp, listPtr,
-		        Tcl_NewStringObj(cmdName, -1));
-            }
-        }
+#ifdef INFO_PROCS_SEARCH_GLOBAL_NS
+	/*
+	 * If "info procs" worked like "info commands", returning the
+	 * commands also seen in the global namespace, then you would
+	 * include this code.  As this could break backwards compatibilty
+	 * with 8.0-8.2, we decided not to "fix" it in 8.3, leaving the
+	 * behavior slightly different.
+	 */
+	if ((nsPtr != globalNsPtr) && !specificNsInPattern) {
+	    entryPtr = Tcl_FirstHashEntry(&globalNsPtr->cmdTable, &search);
+	    while (entryPtr != NULL) {
+		cmdName = Tcl_GetHashKey(&globalNsPtr->cmdTable, entryPtr);
+		if ((simplePattern == NULL)
+	                || Tcl_StringMatch(cmdName, simplePattern)) {
+		    if (Tcl_FindHashEntry(&nsPtr->cmdTable, cmdName) == NULL) {
+			cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+			realCmdPtr = (Command *) TclGetOriginalCommand(
+			        (Tcl_Command) cmdPtr);
+
+			if (TclIsProc(cmdPtr) || ((realCmdPtr != NULL)
+				&& TclIsProc(realCmdPtr))) {
+			    Tcl_ListObjAppendElement(interp, listPtr,
+			            Tcl_NewStringObj(cmdName, -1));
+			}
+		    }
+		}
+		entryPtr = Tcl_NextHashEntry(&search);
+	    }
+	}
+#endif
     }
+
     Tcl_SetObjResult(interp, listPtr);
     return TCL_OK;
 }  
@@ -1503,7 +1583,7 @@ InfoProcsCmd(dummy, interp, objc, objv)
  *          info script
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1543,7 +1623,7 @@ InfoScriptCmd(dummy, interp, objc, objv)
  *          info sharedlibextension
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1581,7 +1661,7 @@ InfoSharedlibCmd(dummy, interp, objc, objv)
  *          info tclversion
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1628,7 +1708,7 @@ InfoTclVersionCmd(dummy, interp, objc, objv)
  *          info vars ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1645,8 +1725,8 @@ InfoVarsCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     Interp *iPtr = (Interp *) interp;
-    char *varName, *pattern;
-    CONST char *simplePattern;
+    char *varName;
+    CONST char *pattern, *simplePattern;
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
     Var *varPtr;
@@ -1956,8 +2036,12 @@ Tcl_LinsertObjCmd(dummy, interp, objc, objv)
      * will invalidate the list's internal representation.
      */
 
-    result = TclGetIntForIndex(interp, objv[2], /*endValue*/ INT_MAX,
-	    &index);
+    result = Tcl_ListObjLength(interp, objv[1], &len);
+    if (result != TCL_OK) {
+	return result;
+    }
+
+    result = TclGetIntForIndex(interp, objv[2], /*endValue*/ len, &index);
     if (result != TCL_OK) {
 	return result;
     }
@@ -2426,7 +2510,7 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument values. */
 {
-    int i, index;
+    int i, index, unique;
     Tcl_Obj *resultPtr;
     int length;
     Tcl_Obj *cmdPtr, **listObjPtrs;
@@ -2435,9 +2519,10 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     SortInfo sortInfo;                  /* Information about this sort that
                                          * needs to be passed to the 
                                          * comparison function */
-    static char *switches[] =
-	    {"-ascii", "-command", "-decreasing", "-dictionary",
-	    "-increasing", "-index", "-integer", "-real", (char *) NULL};
+    static char *switches[] = {
+	"-ascii", "-command", "-decreasing", "-dictionary", "-increasing",
+	"-index", "-integer", "-real", "-unique", (char *) NULL
+    };
 
     resultPtr = Tcl_GetObjResult(interp);
     if (objc < 2) {
@@ -2455,6 +2540,7 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     sortInfo.interp = interp;
     sortInfo.resultCode = TCL_OK;
     cmdPtr = NULL;
+    unique = 0;
     for (i = 1; i < objc-1; i++) {
 	if (Tcl_GetIndexFromObj(interp, objv[i], switches, "option", 0, &index)
 		!= TCL_OK) {
@@ -2504,6 +2590,9 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 	    case 7:			/* -real */
 		sortInfo.sortMode = SORTMODE_REAL;
 		break;
+	    case 8:			/* -unique */
+		unique = 1;
+		break;
 	}
     }
     if (sortInfo.sortMode == SORTMODE_COMMAND) {
@@ -2514,14 +2603,18 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 	 */
 
         Tcl_Obj *newCommandPtr = Tcl_DuplicateObj(cmdPtr);
+	Tcl_Obj *newObjPtr = Tcl_NewObj();
 
-	if (Tcl_ListObjAppendElement(interp, newCommandPtr, Tcl_NewObj()) 
-	        != TCL_OK) {
+	Tcl_IncrRefCount(newCommandPtr);
+	if (Tcl_ListObjAppendElement(interp, newCommandPtr, newObjPtr)
+		!= TCL_OK) {
+	    Tcl_DecrRefCount(newCommandPtr);
+	    Tcl_IncrRefCount(newObjPtr);
+	    Tcl_DecrRefCount(newObjPtr);
 	    return TCL_ERROR;
 	}
 	Tcl_ListObjAppendElement(interp, newCommandPtr, Tcl_NewObj());
 	sortInfo.compareCmdPtr = newCommandPtr;
-	Tcl_IncrRefCount(newCommandPtr);
     }
 
     sortInfo.resultCode = Tcl_ListObjGetElements(interp, objv[objc-1],
@@ -2535,6 +2628,7 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     elementArray = (SortElement *) ckalloc(length * sizeof(SortElement));
     for (i=0; i < length; i++){
 	elementArray[i].objPtr = listObjPtrs[i];
+	elementArray[i].count = 0;
 	elementArray[i].nextPtr = &elementArray[i+1];
     }
     elementArray[length-1].nextPtr = NULL;
@@ -2547,8 +2641,18 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 
 	Tcl_ResetResult(interp);
 	resultPtr = Tcl_GetObjResult(interp);
-	for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
-	    Tcl_ListObjAppendElement(interp, resultPtr, elementPtr->objPtr);
+	if (unique) {
+	    for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
+		if (elementPtr->count == 0) {
+		    Tcl_ListObjAppendElement(interp, resultPtr,
+			    elementPtr->objPtr);
+		}
+	    }
+	} else {
+	    for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
+		Tcl_ListObjAppendElement(interp, resultPtr,
+			elementPtr->objPtr);
+	    }
 	}
     }
     ckfree((char*) elementArray);
@@ -2648,6 +2752,7 @@ MergeLists(leftPtr, rightPtr, infoPtr)
 {
     SortElement *headPtr;
     SortElement *tailPtr;
+    int cmp;
 
     if (leftPtr == NULL) {
         return rightPtr;
@@ -2655,20 +2760,28 @@ MergeLists(leftPtr, rightPtr, infoPtr)
     if (rightPtr == NULL) {
         return leftPtr;
     }
-    if (SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr) > 0) {
+    cmp = SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr);
+    if (cmp > 0) {
 	tailPtr = rightPtr;
 	rightPtr = rightPtr->nextPtr;
     } else {
+	if (cmp == 0) {
+	    leftPtr->count++;
+	}
 	tailPtr = leftPtr;
 	leftPtr = leftPtr->nextPtr;
     }
     headPtr = tailPtr;
     while ((leftPtr != NULL) && (rightPtr != NULL)) {
-	if (SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr) > 0) {
+	cmp = SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr);
+	if (cmp > 0) {
 	    tailPtr->nextPtr = rightPtr;
 	    tailPtr = rightPtr;
 	    rightPtr = rightPtr->nextPtr;
 	} else {
+	    if (cmp == 0) {
+		leftPtr->count++;
+	    }
 	    tailPtr->nextPtr = leftPtr;
 	    tailPtr = leftPtr;
 	    leftPtr = leftPtr->nextPtr;
@@ -2784,10 +2897,10 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 	order = DictionaryCompare(
 		Tcl_GetString(objPtr1),	Tcl_GetString(objPtr2));
     } else if (infoPtr->sortMode == SORTMODE_INTEGER) {
-	int a, b;
+	long a, b;
 
-	if ((Tcl_GetIntFromObj(infoPtr->interp, objPtr1, &a) != TCL_OK)
-		|| (Tcl_GetIntFromObj(infoPtr->interp, objPtr2, &b)
+	if ((Tcl_GetLongFromObj(infoPtr->interp, objPtr1, &a) != TCL_OK)
+		|| (Tcl_GetLongFromObj(infoPtr->interp, objPtr2, &b)
 		!= TCL_OK)) {
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
@@ -2884,7 +2997,7 @@ static int
 DictionaryCompare(left, right)
     char *left, *right;          /* The strings to compare */
 {
-    Tcl_UniChar uniLeft, uniRight;
+    Tcl_UniChar uniLeft, uniRight, uniLeftLower, uniRightLower;
     int diff, zeros;
     int secondaryDiff = 0;
 
@@ -2956,32 +3069,30 @@ DictionaryCompare(left, right)
 	if ((*left != '\0') && (*right != '\0')) {
 	    left += Tcl_UtfToUniChar(left, &uniLeft);
 	    right += Tcl_UtfToUniChar(right, &uniRight);
+	    /*
+	     * Convert both chars to lower for the comparison, because
+	     * dictionary sorts are case insensitve.  Covert to lower, not
+	     * upper, so chars between Z and a will sort before A (where most
+	     * other interesting punctuations occur)
+	     */
+	    uniLeftLower = Tcl_UniCharToLower(uniLeft);
+	    uniRightLower = Tcl_UniCharToLower(uniRight);
 	} else {
 	    diff = UCHAR(*left) - UCHAR(*right);
 	    break;
 	}
 
-        diff = uniLeft - uniRight;
+        diff = uniLeftLower - uniRightLower;
         if (diff) {
+	    return diff;
+	} else if (secondaryDiff == 0) {
 	    if (Tcl_UniCharIsUpper(uniLeft) &&
 		    Tcl_UniCharIsLower(uniRight)) {
-		diff = Tcl_UniCharToLower(uniLeft) - uniRight;
-		if (diff) {
-		    return diff;
-                } else if (secondaryDiff == 0) {
-		    secondaryDiff = -1;
-                }
+		secondaryDiff = -1;
 	    } else if (Tcl_UniCharIsUpper(uniRight)
 		    && Tcl_UniCharIsLower(uniLeft)) {
-                diff = uniLeft - Tcl_UniCharToLower(uniRight);
-                if (diff) {
-		    return diff;
-                } else if (secondaryDiff == 0) {
-		    secondaryDiff = 1;
-                }
-            } else {
-                return diff;
-            }
+		secondaryDiff = 1;
+	    }
         }
     }
     if (diff == 0) {
