@@ -15,11 +15,13 @@
 
 #include "tclInt.h"
 #include "tclPort.h"
+#include "tclCompile.h"
 
 TclStubs *tclStubsPtr = (TclStubs *) NULL;
 TclPlatStubs *tclPlatStubsPtr = (TclPlatStubs *) NULL;
 TclIntStubs *tclIntStubsPtr = (TclIntStubs *) NULL;
 TclIntPlatStubs *tclIntPlatStubsPtr = (TclIntPlatStubs *) NULL;
+TclCompileStubs *tclCompileStubsPtr = (TclCompileStubs *) NULL;
 
 
 /*
@@ -44,8 +46,6 @@ TclIntPlatStubs *tclIntPlatStubsPtr = (TclIntPlatStubs *) NULL;
 #undef Tcl_InitStubs
 #endif
 
-#undef Tcl_InitStubs
-
 char *
 Tcl_InitStubs (interp, version, exact)
     Tcl_Interp *interp;
@@ -72,5 +72,8 @@ Tcl_InitStubs (interp, version, exact)
     tclPlatStubsPtr = tclStubsPtr->hooks->tclPlatStubs;
     tclIntStubsPtr = tclStubsPtr->hooks->tclIntStubs;
     tclIntPlatStubsPtr = tclStubsPtr->hooks->tclIntPlatStubs;
+    if (tclIntStubsPtr->tclFinalizeCondition) {
+        tclCompileStubsPtr = tclStubsPtr->hooks->tclCompileStubs;
+    }
     return actualVersion;
 }
