@@ -21,15 +21,6 @@ extern "C" {
 
 #include "tclInt.h"
 
-/*
- * TCL_STORAGE_CLASS is set unconditionally to DLLEXPORT because the
- * Objtest_Init declaration is in the source file itself, which is only
- * accessed when we are building a library.
- */
-
-#undef TCL_STORAGE_CLASS
-#define TCL_STORAGE_CLASS DLLEXPORT
-
 #ifdef __cplusplus
 }
 #define dummy /* */
@@ -54,8 +45,7 @@ static int		GetVariableIndex _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int *indexPtr));
 static void		SetVarToObj _ANSI_ARGS_((int varIndex,
 			    Tcl_Obj *objPtr));
-EXTERN int		TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
-EXTERN int		Objtest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+int			TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 static int		TestbooleanobjCmd _ANSI_ARGS_((ClientData dummy,
 			    Tcl_Interp *interp, int objc,
 			    Tcl_Obj *CONST objv[]));
@@ -107,17 +97,9 @@ typedef struct TestString {
 int
 TclObjTest_Init(interp)
     Tcl_Interp *interp;
-{
-    return Objtest_Init(interp);
-}
-
-int
-Objtest_Init(interp)
-    Tcl_Interp *interp;
 {
     register int i;
     
-    Tcl_InitStubs(interp, TCL_VERSION, 0);
     for (i = 0;  i < NUMBER_OF_OBJECT_VARS;  i++) {
         varPtr[i] = NULL;
     }
@@ -136,7 +118,7 @@ Objtest_Init(interp)
 	    (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateObjCommand(interp, "teststringobj", TeststringobjCmd,
 	    (ClientData) 0, (Tcl_CmdDeleteProc *) NULL);
-    return Tcl_PkgProvide(interp, "Objtest", TCL_VERSION);
+    return TCL_OK;
 }
 
 /*
