@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclThreadTest.c,v 1.3.4.1 1999/10/30 11:06:01 hobbs Exp $
+ * RCS: @(#) $Id: tclThreadTest.c,v 1.6 2000/01/12 11:12:53 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -265,7 +265,7 @@ Tcl_ThreadObjCmd(dummy, interp, objc, objv)
 	}
 	case THREAD_ID:
 	    if (objc == 2) {
-		Tcl_Obj *idObj = Tcl_NewIntObj((int)Tcl_GetCurrentThread());
+		Tcl_Obj *idObj = Tcl_NewLongObj((long)Tcl_GetCurrentThread());
 		Tcl_SetObjResult(interp, idObj);
 		return TCL_OK;
 	    } else {
@@ -280,7 +280,7 @@ Tcl_ThreadObjCmd(dummy, interp, objc, objv)
 	    return TclThreadList(interp);
 	}
 	case THREAD_SEND: {
-	    int id;
+	    long id;
 	    char *script;
 	    int wait, arg;
 
@@ -299,7 +299,7 @@ Tcl_ThreadObjCmd(dummy, interp, objc, objv)
 		wait = 1;
 		arg = 2;
 	    }
-	    if (Tcl_GetIntFromObj(interp, objv[arg], &id) != TCL_OK) {
+	    if (Tcl_GetLongFromObj(interp, objv[arg], &id) != TCL_OK) {
 		return TCL_ERROR;
 	    }
 	    arg++;
@@ -382,7 +382,7 @@ TclCreateThread(interp, script)
     Tcl_ConditionWait(&ctrl.condWait, &threadMutex, NULL);
     Tcl_MutexUnlock(&threadMutex);
     TclFinalizeCondition(&ctrl.condWait);
-    Tcl_SetObjResult(interp, Tcl_NewIntObj((int)id));
+    Tcl_SetObjResult(interp, Tcl_NewLongObj((long)id));
     return TCL_OK;
 }
 
@@ -502,7 +502,7 @@ ThreadErrorProc(interp)
     Tcl_Channel errChannel;
     char *errorInfo, *script;
     char *argv[3];
-    char buf[10];
+    char buf[TCL_DOUBLE_SPACE+1];
     sprintf(buf, "%ld", (long) Tcl_GetCurrentThread());
 
     errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
@@ -619,7 +619,7 @@ TclThreadList(interp)
     Tcl_MutexLock(&threadMutex);
     for (tsdPtr = threadList ; tsdPtr ; tsdPtr = tsdPtr->nextPtr) {
 	Tcl_ListObjAppendElement(interp, listPtr,
-		Tcl_NewIntObj((int)tsdPtr->threadId));
+		Tcl_NewLongObj((long)tsdPtr->threadId));
     }
     Tcl_MutexUnlock(&threadMutex);
     Tcl_SetObjResult(interp, listPtr);
