@@ -3,7 +3,7 @@
 # utility procs formerly in init.tcl which can be loaded on demand
 # for package management.
 #
-# RCS: @(#) $Id: package.tcl,v 1.5 1999/04/21 21:50:29 rjohnson Exp $
+# RCS: @(#) $Id: package.tcl,v 1.11 2000/02/07 22:33:17 ericm Exp $
 #
 # Copyright (c) 1991-1993 The Regents of the University of California.
 # Copyright (c) 1994-1998 Sun Microsystems, Inc.
@@ -12,6 +12,10 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
 
+# Create the package namespace
+namespace eval ::pkg {
+}
+
 # pkg_compareExtension --
 #
 #  Used internally by pkg_mkIndex to compare the extension of a file to
@@ -32,12 +36,10 @@ proc pkg_compareExtension { fileName {ext {}} } {
     if {[string length $ext] == 0} {
 	set ext [info sharedlibextension]
     }
-    if {[string compare $tcl_platform(platform) "windows"] == 0} {
-	return [expr {[string compare \
-		[string tolower [file extension $fileName]] \
-		[string tolower $ext]] == 0}]
+    if {[string equal $tcl_platform(platform) "windows"]} {
+	return [string equal -nocase [file extension $fileName] $ext]
     } else {
-	return [expr {[string compare [file extension $fileName] $ext] == 0}]
+	return [string equal [file extension $fileName] $ext]
     }
 }
 
@@ -76,7 +78,7 @@ proc pkg_mkIndex {args} {
     }
 
     set more ""
-    set direct 0
+    set direct 1
     set doVerbose 0
     set loadPat ""
     for {set idx 0} {$idx < $argCount} {incr idx} {
@@ -90,8 +92,11 @@ proc pkg_mkIndex {args} {
 	    -verbose {
 		set doVerbose 1
 	    }
+	    -lazy {
+		set direct 0
+		append more " -lazy"
+	    }
 	    -direct {
-		set direct 1
 		append more " -direct"
 	    }
 	    -load {
@@ -115,15 +120,6 @@ proc pkg_mkIndex {args} {
 	set patternList [list "*.tcl" "*[info sharedlibextension]"]
     }
 
-    append index "# Tcl package index file, version 1.1\n"
-    append index "# This file is generated by the \"pkg_mkIndex$more\" command\n"
-    append index "# and sourced either when an application starts up or\n"
-    append index "# by a \"package unknown\" script.  It invokes the\n"
-    append index "# \"package ifneeded\" command to set up package-related\n"
-    append index "# information so that packages will be loaded automatically\n"
-    append index "# in response to \"package require\" commands.  When this\n"
-    append index "# script is sourced, the variable \$dir must contain the\n"
-    append index "# full path name of this file's directory.\n"
     set oldDir [pwd]
     cd $dir
 
@@ -138,7 +134,7 @@ proc pkg_mkIndex {args} {
 	# interpreter, and get a list of the new commands and packages
 	# that are defined.
 
-	if {[string compare $file "pkgIndex.tcl"] == 0} {
+	if {[string equal $file "pkgIndex.tcl"]} {
 	    continue
 	}
 
@@ -156,7 +152,7 @@ proc pkg_mkIndex {args} {
 	    if {! [string match $loadPat [lindex $pkg 1]]} {
 		continue
 	    }
-	    if {[lindex $pkg 1] == "Tk"} {
+	    if {[string equal [lindex $pkg 1] "Tk"]} {
 		$c eval {set argv {-geometry +0+0}}
 	    }
 	    if {[catch {
@@ -165,10 +161,8 @@ proc pkg_mkIndex {args} {
 		if {$doVerbose} {
 		    tclLog "warning: load [lindex $pkg 0] [lindex $pkg 1]\nfailed with: $err"
 		}
-	    } else {
-		if {$doVerbose} {
-		    tclLog "loaded [lindex $pkg 0] [lindex $pkg 1]"
-		}
+	    } elseif {$doVerbose} {
+		tclLog "loaded [lindex $pkg 0] [lindex $pkg 1]"
 	    }
 	}
 	cd $dir
@@ -241,7 +235,7 @@ proc pkg_mkIndex {args} {
 		    return $list
 		}
 
-		# initialize the list of existing namespaces, packages, commands
+		# init the list of existing namespaces, packages, commands
 
 		foreach ::tcl::x [::tcl::GetAllNamespaces] {
 		    set ::tcl::namespaces($::tcl::x) 1
@@ -273,38 +267,46 @@ proc pkg_mkIndex {args} {
 		    set ::tcl::type source
 		}
 
-		# See what new namespaces appeared, and import commands
-		# from them.  Only exported commands go into the index.
-
-		foreach ::tcl::x [::tcl::GetAllNamespaces] {
-		    if {! [info exists ::tcl::namespaces($::tcl::x)]} {
-			namespace import -force ${::tcl::x}::*
-		    }
-		}
-
-		# Figure out what commands appeared
-
-		foreach ::tcl::x [info commands] {
-		    set ::tcl::newCmds($::tcl::x) 1
-		}
-		foreach ::tcl::x $::tcl::origCmds {
-		    catch {unset ::tcl::newCmds($::tcl::x)}
-		}
-		foreach ::tcl::x [array names ::tcl::newCmds] {
-		    # reverse engineer which namespace a command comes from
+		# As a performance optimization, if we are creating 
+		# direct load packages, don't bother figuring out the 
+		# set of commands created by the new packages.  We 
+		# only need that list for setting up the autoloading 
+		# used in the non-direct case.
+		if { !$::tcl::direct } {
+		    # See what new namespaces appeared, and import commands
+		    # from them.  Only exported commands go into the index.
 		    
-		    set ::tcl::abs [namespace origin $::tcl::x]
+		    foreach ::tcl::x [::tcl::GetAllNamespaces] {
+			if {! [info exists ::tcl::namespaces($::tcl::x)]} {
+			    namespace import -force ${::tcl::x}::*
+			}
 
-		    # special case so that global names have no leading
-		    # ::, this is required by the unknown command
-
-		    set ::tcl::abs [auto_qualify $::tcl::abs ::]
-
-		    if {[string compare $::tcl::x $::tcl::abs] != 0} {
-			# Name changed during qualification
-
-			set ::tcl::newCmds($::tcl::abs) 1
-			unset ::tcl::newCmds($::tcl::x)
+			# Figure out what commands appeared
+			
+			foreach ::tcl::x [info commands] {
+			    set ::tcl::newCmds($::tcl::x) 1
+			}
+			foreach ::tcl::x $::tcl::origCmds {
+			    catch {unset ::tcl::newCmds($::tcl::x)}
+			}
+			foreach ::tcl::x [array names ::tcl::newCmds] {
+			    # determine which namespace a command comes from
+			    
+			    set ::tcl::abs [namespace origin $::tcl::x]
+			    
+			    # special case so that global names have no leading
+			    # ::, this is required by the unknown command
+			    
+			    set ::tcl::abs \
+				    [lindex [auto_qualify $::tcl::abs ::] 0]
+			    
+			    if {[string compare $::tcl::x $::tcl::abs]} {
+				# Name changed during qualification
+				
+				set ::tcl::newCmds($::tcl::abs) 1
+				unset ::tcl::newCmds($::tcl::x)
+			    }
+			}
 		    }
 		}
 
@@ -312,7 +314,7 @@ proc pkg_mkIndex {args} {
 		# a version provided, then record it
 
 		foreach ::tcl::x [package names] {
-		    if {([string compare [package provide $::tcl::x] ""] != 0) \
+		    if {[string compare [package provide $::tcl::x] ""] \
 			    && ![info exists ::tcl::packages($::tcl::x)]} {
 			lappend ::tcl::newPkgs \
 			    [list $::tcl::x [package provide $::tcl::x]]
@@ -343,22 +345,33 @@ proc pkg_mkIndex {args} {
 	}
     }
 
+    append index "# Tcl package index file, version 1.1\n"
+    append index "# This file is generated by the \"pkg_mkIndex$more\" command\n"
+    append index "# and sourced either when an application starts up or\n"
+    append index "# by a \"package unknown\" script.  It invokes the\n"
+    append index "# \"package ifneeded\" command to set up package-related\n"
+    append index "# information so that packages will be loaded automatically\n"
+    append index "# in response to \"package require\" commands.  When this\n"
+    append index "# script is sourced, the variable \$dir must contain the\n"
+    append index "# full path name of this file's directory.\n"
+
     foreach pkg [lsort [array names files]] {
-	append index "\npackage ifneeded $pkg "
-	if {$direct} {
-	    set cmdList {}
-	    foreach elem $files($pkg) {
-		set file [lindex $elem 0]
-		set type [lindex $elem 1]
-		lappend cmdList "\[list $type \[file join \$dir\
-			[list $file]\]\]"
-	    }
-	    append index [join $cmdList "\\n"]
-	} else {
-	    append index "\[list tclPkgSetup \$dir [lrange $pkg 0 0]\
-		    [lrange $pkg 1 1] [list $files($pkg)]\]"
+	set cmd {}
+	foreach {name version} $pkg {
+	    break
 	}
+	lappend cmd ::pkg::create -name $name -version $version
+	foreach spec $files($pkg) {
+	    foreach {file type procs} $spec {
+		if { $direct } {
+		    set procs {}
+		}
+		lappend cmd "-$type" [list $file $procs]
+	    }
+	}
+	append index "\n[eval $cmd]"
     }
+
     set f [open pkgIndex.tcl w]
     puts $f $index
     close $f
@@ -412,7 +425,7 @@ proc tclMacPkgSearch {dir} {
 	if {[file isfile $x]} {
 	    set res [resource open $x]
 	    foreach y [resource list TEXT $res] {
-		if {$y == "pkgIndex"} {source -rsrc pkgIndex}
+		if {[string equal $y "pkgIndex"]} {source -rsrc pkgIndex}
 	    }
 	    catch {resource close $res}
 	}
@@ -463,7 +476,8 @@ proc tclPkgUnknown {name version {exact {}}} {
 	# On the Macintosh we also look in the resource fork 
 	# of shared libraries
 	# We can't use tclMacPkgSearch in safe interps because it uses glob
-	if {(![interp issafe]) && ($tcl_platform(platform) == "macintosh")} {
+	if {(![interp issafe]) && \
+		[string equal $tcl_platform(platform) "macintosh"]} {
 	    set dir [lindex $auto_path $i]
 	    tclMacPkgSearch $dir
 	    foreach x [glob -nocomplain [file join $dir *]] {
@@ -475,3 +489,123 @@ proc tclPkgUnknown {name version {exact {}}} {
 	}
     }
 }
+
+# ::pkg::create --
+#
+#	Given a package specification generate a "package ifneeded" statement
+#	for the package, suitable for inclusion in a pkgIndex.tcl file.
+#
+# Arguments:
+#	args		arguments used by the create function:
+#			-name		packageName
+#			-version	packageVersion
+#			-load		{filename ?{procs}?}
+#			...
+#			-source		{filename ?{procs}?}
+#			...
+#
+#			Any number of -load and -source parameters may be
+#			specified, so long as there is at least one -load or
+#			-source parameter.  If the procs component of a 
+#			module specifier is left off, that module will be
+#			set up for direct loading; otherwise, it will be
+#			set up for lazy loading.  If both -source and -load
+#			are specified, the -load'ed files will be loaded 
+#			first, followed by the -source'd files.
+#
+# Results:
+#	An appropriate "package ifneeded" statement for the package.
+
+proc ::pkg::create {args} {
+    append err(usage) "[lindex [info level 0] 0] "
+    append err(usage) "-name packageName -version packageVersion"
+    append err(usage) "?-load {filename ?{procs}?}? ... "
+    append err(usage) "?-source {filename ?{procs}?}? ..."
+
+    set err(wrongNumArgs) "wrong # args: should be \"$err(usage)\""
+    set err(valueMissing) "value for \"%s\" missing: should be \"$err(usage)\""
+    set err(unknownOpt)   "unknown option \"%s\": should be \"$err(usage)\""
+    set err(noLoadOrSource) "at least one of -load and -source must be given"
+
+    # process arguments
+    set len [llength $args]
+    if { $len < 6 } {
+	error $err(wrongNumArgs)
+    }
+    
+    # Initialize parameters
+    set opts(-name)		{}
+    set opts(-version)		{}
+    set opts(-source)		{}
+    set opts(-load)		{}
+
+    # process parameters
+    for {set i 0} {$i < $len} {incr i} {
+	set flag [lindex $args $i]
+	incr i
+	switch -glob -- $flag {
+	    "-name"		-
+	    "-version"		{
+		if { $i >= $len } {
+		    error [format $err(valueMissing) $flag]
+		}
+		set opts($flag) [lindex $args $i]
+	    }
+	    "-source"		-
+	    "-load"		{
+		if { $i >= $len } {
+		    error [format $err(valueMissing) $flag]
+		}
+		lappend opts($flag) [lindex $args $i]
+	    }
+	    default {
+		error [format $err(unknownOpt) [lindex $args $i]]
+	    }
+	}
+    }
+
+    # Validate the parameters
+    if { [llength $opts(-name)] == 0 } {
+	error [format $err(valueMissing) "-name"]
+    }
+    if { [llength $opts(-version)] == 0 } {
+	error [format $err(valueMissing) "-version"]
+    }
+    
+    if { [llength $opts(-source)] == 0 && [llength $opts(-load)] == 0 } {
+	error $err(noLoadOrSource)
+    }
+
+    # OK, now everything is good.  Generate the package ifneeded statment.
+    set cmdline "package ifneeded $opts(-name) $opts(-version) "
+    
+    set cmdList {}
+    set lazyFileList {}
+
+    # Handle -load and -source specs
+    foreach key {load source} {
+	foreach filespec $opts(-$key) {
+	    foreach {filename proclist} {{} {}} {
+		break
+	    }
+	    foreach {filename proclist} $filespec {
+		break
+	    }
+	    
+	    if { [llength $proclist] == 0 } {
+		set cmd "\[list $key \[file join \$dir [list $filename]\]\]"
+		lappend cmdList $cmd
+	    } else {
+		lappend lazyFileList [list $filename $key $proclist]
+	    }
+	}
+    }
+
+    if { [llength $lazyFileList] > 0 } {
+	lappend cmdList "\[list tclPkgSetup \$dir $opts(-name)\
+		$opts(-version) [list $lazyFileList]\]"
+    }
+    append cmdline [join $cmdList "\\n"]
+    return $cmdline
+}
+
