@@ -7,7 +7,7 @@
  * Copyright (c) 1999 by Scriptics Corporation.
  * All rights reserved.
  *
- * RCS: @(#) $Id: tclUnixInit.c,v 1.17 1999/07/22 21:50:56 redman Exp $
+ * RCS: @(#) $Id: tclUnixInit.c,v 1.18 1999/10/13 00:32:49 hobbs Exp $
  */
 
 #include "tclInt.h"
@@ -342,44 +342,6 @@ CONST char *path;		/* Path to the executable in native
     TclSetLibraryPath(pathPtr);    
     Tcl_DStringFree(&buffer);
 }
-
-
-static Tcl_Obj *startupLocale = NULL;
-static Tcl_Obj *startupEncoding = NULL;
-
-void
-TclpGetStartupLocaleEncoding(encodingPtr, localePtr)
-     Tcl_Obj **encodingPtr;
-     Tcl_Obj **localePtr;
-{
-    if (encodingPtr != NULL) {
-	*encodingPtr = startupEncoding;
-    }
-    if (localePtr != NULL) {
-	*localePtr = startupLocale;
-    }
-}
-
-static void	SetStartupLocaleEncoding _ANSI_ARGS_((char *encoding, char *locale));
-void
-SetStartupLocaleEncoding(encoding, locale)
-     char *encoding;
-     char *locale;
-{
-    if (!startupLocale) {
-	if (encoding == NULL || encoding[0] == '\0') {
-	    Tcl_Panic("what's up?? can't get initial encoding??");
-	}
-	if (locale == NULL || locale[0] == '\0') {
-	    locale = "C";
-	}
-	startupEncoding = Tcl_NewStringObj(encoding,-1);
-	Tcl_IncrRefCount(startupEncoding);
-	startupLocale = Tcl_NewStringObj(locale,-1);
-	Tcl_IncrRefCount(startupLocale);
-    }
-}
-
 
 /*
  *---------------------------------------------------------------------------
@@ -455,7 +417,6 @@ TclpSetInitialEncodings()
 		encoding = Tcl_DStringValue(&ds);
 		Tcl_UtfToLower(Tcl_DStringValue(&ds));
 		if (Tcl_SetSystemEncoding(NULL, encoding) == TCL_OK) {
-		    SetStartupLocaleEncoding((char *)encoding, langEnv);
 		    Tcl_DStringFree(&ds);
 		    goto resetPath;
 		}
@@ -470,8 +431,6 @@ TclpSetInitialEncodings()
 
     Tcl_SetSystemEncoding(NULL, encoding);
 
-    SetStartupLocaleEncoding((char *)encoding, langEnv);
-
     /*
      * Hey, Belows are helpful for X11 XIM, NOT!!!!
      * If setlocale(LC_ALL, "") are once called, NOT reset to "C"
@@ -578,8 +537,6 @@ TclpSetVariables(interp)
     int unameOK;
     char *user;
     Tcl_DString ds;
-    Tcl_Obj *encoding;
-    Tcl_Obj *locale;
 
     Tcl_SetVar(interp, "tclDefaultLibrary", defaultLibraryDir, TCL_GLOBAL_ONLY);
     Tcl_SetVar(interp, "tcl_pkgPath", pkgPath, TCL_GLOBAL_ONLY);
@@ -637,16 +594,9 @@ TclpSetVariables(interp)
 	    user = "";
 	}
     }
+    Tcl_SetVar2(interp, "tcl_platform", "user", user, TCL_GLOBAL_ONLY);
     Tcl_DStringFree(&ds);
 
-    Tcl_SetVar2(interp, "tcl_platform", "user", user, TCL_GLOBAL_ONLY);
-    TclpGetStartupLocaleEncoding(&encoding, &locale);
-    if (encoding) {
-	Tcl_SetVar2Ex(interp, "tcl_platform", "encoding", encoding, TCL_GLOBAL_ONLY);
-    }
-    if (locale) {
-	Tcl_SetVar2Ex(interp, "tcl_platform", "locale", locale, TCL_GLOBAL_ONLY);
-    }
 }
 
 /*
@@ -655,7 +605,7 @@ TclpSetVariables(interp)
  * TclpFindVariable --
  *
  *	Locate the entry in environ for a given name.  On Unix this 
- *	routine is case sensetive, on Windows this matches mioxed case.
+ *	routine is case sensetive, on Windows this matches mixed case.
  *
  * Results:
  *	The return value is the index in environ of an entry with the
