@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinDde.c,v 1.2 1999/04/16 00:48:08 stanton Exp $
+ * RCS: @(#) $Id: tclWinDde.c,v 1.5 1999/06/26 22:41:53 redman Exp $
  */
 
 #include "tclPort.h"
@@ -64,11 +64,15 @@ static Tcl_ThreadDataKey dataKey;
  * The following variables cannot be placed in thread-local storage.
  * The Mutex ddeMutex guards access to the ddeInstance.
  */
-static HSZ ddeService_g = 0;
-static DWORD ddeInstance = 0;       /* The application instance handle given
+static HSZ ddeServiceGlobal = 0;
+static DWORD ddeInstance;       /* The application instance handle given
 				 * to us by DdeInitialize. */
 static int ddeIsServer = 0;
 
+#define TCL_DDE_VERSION "1.1"
+#define TCL_DDE_PACKAGE_NAME "dde"
+#define TCL_DDE_SERVICE_NAME "TclEval"
+
 TCL_DECLARE_MUTEX(ddeMutex)
 
 /*
@@ -132,9 +136,25 @@ Dde_Init(
     }
     Tcl_CreateExitHandler(DdeExitProc, NULL);
 
-    return Tcl_PkgProvide(interp, "dde", "1.1");
+    return Tcl_PkgProvide(interp, TCL_DDE_PACKAGE_NAME, TCL_DDE_VERSION);
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Initialize --
+ *
+ *	Initialize the global DDE instance.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Registers the DDE server proc.
+ *
+ *----------------------------------------------------------------------
+ */
 
 static void
 Initialize(void)
@@ -170,20 +190,20 @@ Initialize(void)
 	}
 	Tcl_MutexUnlock(&ddeMutex);
     }
-    if ((ddeService_g == 0) && (nameFound != 0)) {
+    if ((ddeServiceGlobal == 0) && (nameFound != 0)) {
 	Tcl_MutexLock(&ddeMutex);
-	if ((ddeService_g == 0) && (nameFound != 0)) {
+	if ((ddeServiceGlobal == 0) && (nameFound != 0)) {
 	    ddeIsServer = 1;
 	    Tcl_CreateExitHandler(DdeExitProc, NULL);
-	    ddeService_g = DdeCreateStringHandle(ddeInstance, "TclEval", 0);
-	    DdeNameService(ddeInstance, ddeService_g, 0L, DNS_REGISTER);
+	    ddeServiceGlobal = DdeCreateStringHandle(ddeInstance, \
+		    TCL_DDE_SERVICE_NAME, 0);
+	    DdeNameService(ddeInstance, ddeServiceGlobal, 0L, DNS_REGISTER);
 	} else {
 	    ddeIsServer = 0;
 	}
 	Tcl_MutexUnlock(&ddeMutex);
     }
 }    
-
 
 /*
  *--------------------------------------------------------------
@@ -211,10 +231,13 @@ Initialize(void)
  */
 
 static char *
-DdeSetServerName(Tcl_Interp *interp, char *name)
-   /* "name" will be used to refer to the interpreter in later "send" commands.
-    *  It must be globally unique.
-    */			
+DdeSetServerName(
+    Tcl_Interp *interp,
+    char *name			/* The name that will be used to
+				 * refer to the interpreter in later
+				 * "send" commands.  Must be globally
+				 * unique. */
+    )
 {
     int suffix, offset;
     RegisteredInterp *riPtr, *prevPtr;
@@ -287,6 +310,11 @@ DdeSetServerName(Tcl_Interp *interp, char *name)
     }
     Tcl_DStringFree(&dString);
 
+    /*
+     * re-initialize with the new name
+     */
+    Initialize();
+    
     return riPtr->name;
 }
 
@@ -653,7 +681,6 @@ DdeServerProc (
     }
     return NULL;
 }
-
 
 /*
  *--------------------------------------------------------------
@@ -675,8 +702,6 @@ static void
 DdeExitProc(
     ClientData clientData)	    /* Not used in this handler. */
 {
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-
     DdeNameService(ddeInstance, NULL, 0, DNS_UNREGISTER);
     DdeUninitialize(ddeInstance);
     ddeInstance = 0;
@@ -753,7 +778,6 @@ SetDdeError(
 {
     Tcl_Obj *resultPtr = Tcl_GetObjResult(interp);
     int err;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
     err = DdeGetLastError(ddeInstance);
     switch (err) {
@@ -884,15 +908,14 @@ Tcl_DdeObjCmd(
 		firstArg = 3;
 	    }
 	    break;
-
-	case DDE_POKE:
-		if (objc != 6) {
-			Tcl_WrongNumArgs(interp, 1, objv,
-      					"poke serviceName topicName item data_value");
-			return TCL_ERROR;
-		}
-		firstArg = 2;
-		break;
+ 	case DDE_POKE:
+	    if (objc != 6) {
+		Tcl_WrongNumArgs(interp, 1, objv,
+			"poke serviceName topicName item value");
+		return TCL_ERROR;
+	    }
+	    firstArg = 2;
+	    break;
 
 	case DDE_REQUEST:
 	    if (objc != 5) {
@@ -952,7 +975,7 @@ Tcl_DdeObjCmd(
 		CP_WINANSI);
     }
 
-    if ((index != DDE_SERVERNAME) && (index != DDE_EVAL)) {
+    if ((index != DDE_SERVERNAME) &&(index != DDE_EVAL)) {
 	topicName = Tcl_GetStringFromObj(objv[firstArg + 1], &length);
 	if (length == 0) {
 	    topicName = NULL;
@@ -983,8 +1006,8 @@ Tcl_DdeObjCmd(
 	    }
 	    hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, 
                     NULL);
-		DdeFreeStringHandle (ddeInstance, ddeService) ;
-		DdeFreeStringHandle (ddeInstance, ddeTopic) ;
+	    DdeFreeStringHandle (ddeInstance, ddeService) ;
+	    DdeFreeStringHandle (ddeInstance, ddeTopic) ;
 
 	    if (hConv == NULL) {
 		SetDdeError(interp);
@@ -1015,37 +1038,6 @@ Tcl_DdeObjCmd(
 	    }
 	    break;
 	}
-	case DDE_POKE: {
-		itemString = Tcl_GetStringFromObj(objv[firstArg + 2], &length);
-		if (length == 0) {
-			Tcl_SetStringObj(Tcl_GetObjResult(interp),
-											"cannot have a null item", -1);
-			return TCL_ERROR;
-		}
-		dataString = Tcl_GetStringFromObj(objv[firstArg + 3], &length);
-
-		hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
-		DdeFreeStringHandle (ddeInstance,ddeService) ;
-		DdeFreeStringHandle (ddeInstance, ddeTopic) ;
-		if (hConv == NULL) {
-			SetDdeError(interp);
-			result = TCL_ERROR;
-		} else {
-			ddeItem = DdeCreateStringHandle(ddeInstance, itemString, CP_WINANSI);
-			if (ddeItem != NULL) {
-				ddeData = DdeClientTransaction(dataString,length+1, hConv, ddeItem,
-			    										CF_TEXT, XTYP_POKE, 5000, NULL);
-				if (ddeData == NULL) {
-					SetDdeError(interp);
-					result = TCL_ERROR;
-		    	}
-			} else {
-				SetDdeError(interp);
-				result = TCL_ERROR;
-			}
-      }
-      break;
-	}
 	case DDE_REQUEST: {
 	    itemString = Tcl_GetStringFromObj(objv[firstArg + 2], &length);
 	    if (length == 0) {
@@ -1053,10 +1045,9 @@ Tcl_DdeObjCmd(
 			"cannot request value of null data", -1);
 		return TCL_ERROR;
 	    }
-	    hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, 
-                    NULL);
-		DdeFreeStringHandle (ddeInstance, ddeService) ;
-		DdeFreeStringHandle (ddeInstance, ddeTopic) ;
+	    hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
+	    DdeFreeStringHandle (ddeInstance, ddeService) ;
+	    DdeFreeStringHandle (ddeInstance, ddeTopic) ;
 	    
 	    if (hConv == NULL) {
 		SetDdeError(interp);
@@ -1086,6 +1077,41 @@ Tcl_DdeObjCmd(
 
 	    break;
 	}
+	case DDE_POKE: {
+	    itemString = Tcl_GetStringFromObj(objv[firstArg + 2], &length);
+	    if (length == 0) {
+		Tcl_SetStringObj(Tcl_GetObjResult(interp),
+			"cannot have a null item", -1);
+		return TCL_ERROR;
+	    }
+	    dataString = Tcl_GetStringFromObj(objv[firstArg + 3], &length);
+	    
+	    hConv = DdeConnect(ddeInstance, ddeService, ddeTopic, NULL);
+	    DdeFreeStringHandle (ddeInstance,ddeService) ;
+	    DdeFreeStringHandle (ddeInstance, ddeTopic) ;
+
+	    if (hConv == NULL) {
+		SetDdeError(interp);
+		result = TCL_ERROR;
+	    } else {
+		ddeItem = DdeCreateStringHandle(ddeInstance, itemString, \
+			CP_WINANSI);
+		if (ddeItem != NULL) {
+		    ddeData = DdeClientTransaction(dataString,length+1, \
+			    hConv, ddeItem,
+			    CF_TEXT, XTYP_POKE, 5000, NULL);
+		    if (ddeData == NULL) {
+			SetDdeError(interp);
+			result = TCL_ERROR;
+		    }
+		} else {
+		    SetDdeError(interp);
+		    result = TCL_ERROR;
+		}
+	    }
+	    break;
+	}
+
 	case DDE_SERVICES: {
 	    HCONVLIST hConvList;
 	    CONVINFO convInfo;
@@ -1096,9 +1122,8 @@ Tcl_DdeObjCmd(
 	    convInfo.cb = sizeof(CONVINFO);
 	    hConvList = DdeConnectList(ddeInstance, ddeService, 
                     ddeTopic, 0, NULL);
-		DdeFreeStringHandle (ddeInstance,ddeService) ;
-		DdeFreeStringHandle (ddeInstance, ddeTopic) ;
-
+	    DdeFreeStringHandle (ddeInstance,ddeService) ;
+	    DdeFreeStringHandle (ddeInstance, ddeTopic) ;
 	    hConv = 0;
 	    convListObjPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
 	    Tcl_DStringInit(&dString);
@@ -1233,7 +1258,7 @@ Tcl_DdeObjCmd(
 		
 		if (ddeData == 0) {
 		    SetDdeError(interp);
-		    goto error1;
+		    goto errorNoResult;
 		}
 		
 		if (async == 0) {
@@ -1306,7 +1331,8 @@ Tcl_DdeObjCmd(
     error:
     Tcl_SetStringObj(Tcl_GetObjResult(interp),
 	    "invalid data returned from server", -1);
-error1:
+
+    errorNoResult:
     if (ddeCookie != NULL) {
 	DdeFreeStringHandle(ddeInstance, ddeCookie);
     }
