@@ -35,18 +35,6 @@ DllEntryPoint(
 {
     switch (reason) {
     case DLL_PROCESS_ATTACH:
-	if (TclWinGetTclInstance() != NULL) {
-	    /*
-	     * Prevents DLL from being loaded multiple times under Win32s,
-	     * since all copies of the DLL share the same data segment and
-	     * Tcl isn't set up to handle that.  Under NT or 95, each time 
-	     * the DLL is loaded, it gets its own private copy of the data 
-	     * segment.
-	     */
-
-	    return FALSE;
-	}
-
 	TclWinInit(hInst);
 	return TRUE;
 
